#!/usr/local/bin/perl
#
# $Id: disk-larrd.pl,v 1.5 2004/02/24 20:30:11 cncook Exp $
#
# disk-larrd.pl: larrd module to collect disk utilization stats 
#
# Scott Walters Copyright 1999
# scott@PacketPushers.com 
#  
# Christian Pearce
# pearcec@PacketPushers.com
#
# Craig Cook 13 Mar 2003        http://www.cookitservices.com
# Created @ds
# Created $ds
#
# Do whatever you want with it
# just don't sell it and make money off of it and not cut me in ;)
# I take no liability for it, use at your own risks

# 27 Jun 2003   Craig Cook      http://www.cookitservices.com
# Changed status check to look for colour status only at start of line

# 8 Feb 2005   Craig Cook      http://www.cookitservices.com
# Now uses procedures
# Convert kilobytes to bytes
# Note: this will mess up any current disk graphs you have
# Added storage of available space and total disk space
# Note: you will have to convert or delete every disk graph you have
# We capture all information, since calculating the values yourself
#   does not seem to be accurate.
#
# 16 Feb 2005   Craig Cook      http://www.cookitservices.com
# Added variable $all_disk_graphs to allow older larrd functionality
# Now uses LARRD.pm

$DEBUG = 0;
$DEBUG = 1 if $ENV{'DEBUG'};
$WARN = 1 if $ENV{'WARN'};

if ($DEBUG) {
  $WARN = 1;
  $ENV{'BBHOME'}="/usr/local/bb";
  $ENV{'BBVAR'}="/usr/local/bbvar";
  $ENV{'BBLOGS'}="/usr/local/bbvar/logs";
  $ENV{'BBRRDS'}="$ENV{'BBVAR'}/rrd";
  use lib "/usr/local/rrdtool/lib/perl/";
  use lib "/usr/local/larrd/lib/LARRD";
}


# Set a path to the rrd tool lib
# Set as and $ENV and see what is up
use lib $ENV{'RRDPM'};
use RRDs;

use lib $ENV{'LARRDPM'};
use LARRD;

my $larrd = new LARRD;

my $all_disk_graphs = "yes"; # change to "no" to restore 0.43c and older larrd graphs

my $ds = "disk";

if ("$all_disk_graphs" eq "yes") {

@ds = ("DS:pct:GAUGE:600:0:100",
       "DS:used:GAUGE:600:0:U",
       "DS:avail:GAUGE:600:0:U",
       "DS:total:GAUGE:600:0:U"
      );

} else {
  
@ds = ("DS:pct:GAUGE:600:0:100",
       "DS:used:GAUGE:600:0:U",
      );

}

my $reference_to_rrd_data_structure = \@ds;

my $one_kilobyte = 1024;

##################################
# Start of script
##################################

$larrd->check_larrd_env;

##
# real work begins here
##

# We only look in the Big Brother logs dir, disk data is only sent there, not to bbvar/data

$filenames_ref = $larrd->get_bb_log_filenames($DEBUG);

$hash_ref_for_hosts = $larrd->read_bb_logs($DEBUG,$ds,$filenames_ref);

while ( ($host,$log_filename) = each %$hash_ref_for_hosts ) {


  print "$0: DEBUG host: $host, log: $log_filename\n" if ($DEBUG) ;

    $log_time = $larrd->get_log_time($DEBUG,$log_filename);

    $reference_to_log_data = $larrd->get_info_from_log($DEBUG,$log_filename);

    if($$reference_to_log_data[0] !~ /^(green|yellow|red)\s+/) {
            print "$0: WARN $host is not showing red, yellow, or green. $log skipped.\n" if ($WARN);
            next; #next LOG;
    }

    # Now lets process the data

    my ($RRD,$data_value) = &process_log($DEBUG,$reference_to_log_data);

    #&update_rrd($DEBUG,$RRD,$log_time,$data_value);

} # end while


##################################
# Start of Subprocedures
##################################

sub process_log ($$) {

my $DEBUG = shift;
my $log_data_ref = shift;
my $systemtype = undef;
my $diskname;
my $pct;
my $used;
my $avail;
my $total;
my @col;

  print "$0: DEBUG in process_log\n" if $DEBUG;

	# Ugly I know, but effiecent
	my $whole = join "",@$log_data_ref;

	# Decide the client system type
	if($whole =~ / xfs | efs | cxfs / ) {
		$systemtype="IRIX";
        } elsif ($whole =~ /DASD/ ) { 
		$systemtype="AS400";
	} elsif ($whole =~ /Filesystem/ ) { # Hope this doesn't break
		# It seems that UNIX does not have "Filesystem" in the LOG
		$systemtype="NT";
	} else {
		$systemtype="UNIX";
	}

  foreach my $line (@$log_data_ref) {

                # Skip blank lines
		next if ($line =~/^$/);

                # Skip header line for NT sytems
		next if ($line =~ /^Filesystem/);
		next if ($line =~ /^Status/);
                
                # AS400 output looks like this:
                # green July 15, 2003 7:54:25 AM EDT DASD on as400 OK at 86.0683%
		next if ($line =~ /\d\d\:\d\d\:\d\d/ && $systemtype ne "AS400");

                # We only want the DASD line for AS400
		next if ($line !~ /DASD/ && $systemtype eq "AS400" );

                # No idea what system uses the word bloater - better leave it here though - Craig Cook
		last if ($line =~ /bloater/);

		print "$0: DEBUG systemtype: $systemtype, line: $line\n" if $DEBUG;

		@col = split " ",$line;

		# sometimes NT is wierd
                # what am i saying, it's always wierd.
                # can i waste any more CPU cycles?
		if ( $systemtype eq "AS400" ) {
			$diskname=",DASD"; 
			$col12 =~ s/%//g;
			$pct=$col[12];
			$used=0;	
                        $avail=0;
                        $total=0;
		} elsif ( $systemtype eq "NT" ) {
			$diskname="\,$col[0]";
		        $col[4] =~ s/%//g;
			$pct=$col[4];
                	$used=$col[2];
                        $used=$used*$one_kilobyte; # convert kilobytes to bytes
                        $avail=$col[3];
                        $avail=$avail*$one_kilobyte; # convert kilobytes to bytes
                        $total=$col[1];
                        $total=$total*$one_kilobyte; # convert kilobytes to bytes
		} elsif ( $systemtype eq "IRIX" ) {
		        # /'s are a nasty thing to have in filenames
		        $col[6] =~ s/\//\,/g;
			$diskname=$col[6];
		        $col[5] =~ s/%//g;
			$pct=$col[5];
                	$used=$col[3];
                        $used=$used*$one_kilobyte; # convert kilobytes to bytes
                	$avail=$col[4];
                        $avail=$avail*$one_kilobyte; # convert kilobytes to bytes
                	$total=$col[2];
                        $total=$total*$one_kilobyte; # convert kilobytes to bytes
		} else {
		        # /'s are a nasty thing to have in filenames
		        $col[5] =~ s/\//\,/g;
			$diskname=$col[5]; 
		        $col[4] =~ s/%//g;
			$pct=$col[4];
                	$used=$col[2];
                        $used=$used*$one_kilobyte; # convert kilobytes to bytes
                        $avail=$col[3];
                        $avail=$avail*$one_kilobyte; # convert kilobytes to bytes
                        $total=$col[1];
                        $total=$total*$one_kilobyte; # convert kilobytes to bytes
		}

		# want / filesytems to have nice name
                if ( $diskname eq "\," ) {
                        $diskname="\,root"
		}

                $RRD="$ENV{'BBRRDS'}/$host.$ds$diskname.rrd";

                if ($all_disk_graphs eq "yes") {
                  $data_value = "$pct:$used:$avail:$total";
                } else {
                  $data_value ="$pct:$used";
                }

                print "RRD: $RRD, all_disk_graphs: $all_disk_graphs, data_value: $data_value\n" if $DEBUG;

                # This is poor, I should pass results out and call &update_rrd from the while loop	
                $larrd->update_rrd($DEBUG,$RRD,$log_time,$data_value,$reference_to_rrd_data_structure);
                #return ($RRD,$data_value);

	}

} # end sub process_log


##############################################
##############################################
