#!/usr/local/bin/perl
#
# $Id: la-larrd.pl,v 1.8 2004/02/26 21:40:25 cncook Exp $
#
# la-larrd.pl - larrd module for collecting Load Average stats 
#
# Scott Walters Copyright 1999
# scott@PacketPushers.com 
#
# Christian Pearce
# pearcec@PacketPushers.com
#  
# Version 1.1   Craig Cook   29 Sept 2001
#            Added cacheflow cpu check
#         1.2   Craig Cook   1 Aug 2002
#            Allow for DISPREALLOADAVG=TRUE
#            Use 5 min value if available
#         1.3 added create_rrd procedure
#             applied patch by Peter Smith
#             - could fail colour check
#             updated patch by Peter

# 27 Jun 2003   Craig Cook
# Changed status check to look for colour status only at start of line
#
# Do whatever you want with it
# just don't sell it and make money off of it and not cut me in ;)
# I take no liability for it, use at your own risks

# Set a path to the rrd tool lib
# Set as an $ENV and see what is up
use lib $ENV{'RRDPM'};
#use lib "/usr/local/rrdtool/lib/perl/";
use RRDs;

use lib $ENV{'LARRDPM'};
#use lib "/usr/local/larrd/lib/LARRD";
use LARRD;
#use strict;

my $larrd = new LARRD;

my $ds = "cpu";

my @ds = ("DS:la:GAUGE:600:0:U",
         );

my $DEBUG = 1 if $ENV{'DEBUG'};
my $WARN = 1 if $ENV{'WARN'};
my $log;
my $host;
my $colour_check;

##################################
# Start of script
##################################

$larrd->check_larrd_env;

##
# real work begins here
##

# go to the logs area for simplification of the list
chdir "$ENV{'BBLOGS'}";

# Glob the la logs
LOG: while (  defined ($log = <*.$ds>)  ) {

        $host=$log;
        # create some handy vars
        $host =~ s/\.$ds//;
        $host =~ s/\,/\./g;

        &get_log_time;

        $colour_check="notok";

	# Grab Load Avg line
        open(LOG,"$log");
        #while ( defined ($line = <LOG>) && $colour_check eq "ok" ) {
        while ( defined ($line = <LOG>) ) {
	  chomp $line;

          next if ($line =~ /^\s*$/);   # Skip blank lines
          next if ($line =~ /^Status/);   # Skip Status lines

          print "$0: DEBUG $host line: $line\n" if $DEBUG; 
          # ensure colour is valid - ie don't graph purple results
	  if($line =~ /^green\s+|^yellow\s+|^red\s+/) {
  	    # New attempt at grabing procs and users.
	    $line =~ /(\d+)\s+users,\s+(\d+)\s+procs/;
	    $users=$1;
	    $procs=$2;
            $colour_check="ok";

	    if($line =~ /%/) {
	      # This might break NT, but hope it didn't lose
	      # anything in the translation of Andreas Holzhammer fix
	      $line =~ s/[^%]*[=]([0-9][0-9]*)%.*/$1/;
              # We only get here if we have a valid cpu %
	      $la=$line;
  	    } else {
		($dummy,$line) = split /=/,$line,3;	
              # We only get here if we have a valid cpu %
                if ($line =~ /\./) { # ie DISPREALLOADAVG=TRUE
                  $line = $line * 100 ;
                }
	        $la=$line;
	    } # end if($line =~ /%/)

          } elsif ($line =~ /CPU usage/) {
              $line =~ s/CPU usage:\s+(\d+)%[^,]+,\s*(\d+)%/$1,$2/;
              # We only get here if we have a valid cpu %
              # Use the 5min avg if possible, else use what we find
              $la_1min=$1;
              $la_5min=$2;
              if ( defined $la_5min ) {
                   $la=$la_5min;
              } else {
                     $la=$la_1min;
              }
          } # end line colour check

          if ($colour_check ne "ok") {
		print "$0: WARN $host is not showing red, yellow, or green. $log skipped.\n" if ($WARN);
		next LOG;
	  }

        } # end while	
	close(LOG);

        print "$0: DEBUG $host users: $users, procs: $procs, la: $la\n" if $DEBUG; 
	
	# RRD for la
	
	$RRD="$ENV{'BBRRDS'}/$host.la.rrd";

	# if it doesn't exist, make it 
	if ( not -f $RRD ) {
		&create_rrd;
	}

	RRDs::update("$RRD","$ctime:$la");
        $ERR=RRDs::error;
        if($ERR) {
 	       print "$0: WARN updating $RRD: $ERR\n" if ($WARN);
        }

	# Sloppy
        &get_log_time;

	# RRD for users

#
# Users
# 

      if (defined $users) {
	$RRD="$ENV{'BBRRDS'}/$host.users.rrd";

	# if it doesn't exist, make it 
	if ( not -f $RRD ) {
		&create_rrd;
	}

	RRDs::update("$RRD","$ctime:$users");
        $ERR=RRDs::error;
        if($ERR) {
 	       print "$0: WARN updating $RRD: $ERR\n" if ($WARN);
        }

     } # end if (defined $users)

     #
     # Procs
     # 
      if (defined $procs) {

	# Sloppy
        &get_log_time;

	# RRD for procs
	
	$RRD="$ENV{'BBRRDS'}/$host.procs.rrd";

	# if it doesn't exist, make it 
	if ( not -f $RRD ) {
		&create_rrd;
	}

	RRDs::update("$RRD","$ctime:$procs");
        $ERR=RRDs::error;
        if($ERR) {
 	       print "$0: WARN updating $RRD: $ERR\n" if ($WARN);
        }
 
      } # end if (defined $procs) {

}


sub create_rrd {

  print "$0: DEBUG in create_rrd\n" if $DEBUG;

    # RRAS defined in larrd.pl
    @rras = split " ",$ENV{'RRAS'};

    RRDs::create($RRD,@ds,@rras);
    $ERR=RRDs::error;
    if($ERR) {
      print "$0: ERROR creating $RRD: $ERR\n" if ($ENV{'ERROR'});
      next;
    }

    $ctime="N";
    print "$0: STATUS did not find $RRD, created.\n" if ($ENV{'STATUS'});
}

sub get_log_time {
  # Sometimes you get it from stat, sometimes you get it from the log
  ($ctime) = (stat($log))[10];
  print "$0 : DEBUG ctime = $ctime\n" if ($DEBUG);
}

##############################################
# end of script
##############################################
