#!/usr/local/bin/perl
#
# up-larrd.pl - larrd module for collecting system uptime
#               - how long has each host been running?
#               - you will clearly see reboots over time!
#
# Do whatever you want with it, just don't sell it!
# I take no liability for it, use at your own risk!

# V1.0 14 Feb 2005    Craig Cook
#

# Todo: make this work with cpu uptime stats
# Note: this will require a patch to bb-cpu.sh
# Look for:
#        UP=`echo $3 $4 | $SED "s/,//"`
# Replace with:
#        UP=`echo $3 $4 $5 | $SED "s/,//g"`

my $DEBUG = 0;
$DEBUG = 1 if $ENV{'DEBUG'};

if ($DEBUG) {
  $WARN = 1;
  use Data::Dumper;
  $ENV{'BBHOME'}="/usr/local/bb";
  $ENV{'BBVAR'}="/usr/local/bbvar";
  $ENV{'BBLOGS'}="/usr/local/bbvar/logs";
  $ENV{'BBRRDS'}="$ENV{'BBVAR'}/rrd";
  use lib "/usr/local/rrdtool/lib/perl/";
  use lib "/usr/local/larrd/lib/LARRD";
}


# Set a path to the rrd tool lib
# Set as and $ENV and see what is up
use lib $ENV{'RRDPM'};
use RRDs;

use lib $ENV{'LARRDPM'};
use LARRD;

my $larrd = new LARRD;

my $ds = "up";

my @ds = ("DS:minutes:GAUGE:600:0:U");
my $reference_to_rrd_data_structure = \@ds;
my $RRD;

##################################
# Start of script
##################################

$larrd->check_larrd_env;

##
# real work begins here
##

$filenames_ref = $larrd->get_bb_log_filenames($DEBUG);

$hash_ref_for_hosts = $larrd->read_bb_logs($DEBUG,$ds,$filenames_ref);

while ( ($host,$log_filename) = each %$hash_ref_for_hosts ) {

    $RRD="$ENV{'BBRRDS'}/$host.$ds.rrd";

    print "$0 : DEBUG host=$host, log=$log_filename\n" if ($DEBUG);

    $log_time = $larrd->get_log_time($DEBUG,$log_filename);

    $reference_to_log_data = $larrd->get_info_from_log($DEBUG,$log_filename);

    if($$reference_to_log_data[0] !~ /^(green|yellow|red)\s+/) {
            print "$0: WARN $host is not showing red, yellow, or green. $log skipped.\n" if ($WARN);
            next;
    }

    # Now lets process the data

    my $data_value = &process_log($DEBUG,$reference_to_log_data);

    if ($data_value ne "") {

       # If we are looking at a log with no larrdable data
       # do not create the rrd and in turn the graph
       # It makes sense this way.

       $larrd->update_rrd($DEBUG,$RRD,$log_time,$data_value,$reference_to_rrd_data_structure);

    } # end if ($data_value ne "") {


}

sub process_log ($$) {
my $DEBUG = shift;
my $log_data_ref = shift;
my $days_up = undef;
my $hours_up = undef;
my $minutes_up = undef;
my $seconds_up = undef;
my $hundreth_seconds_up = undef;

  print "$0: DEBUG in process_log\n" if $DEBUG;

  foreach $line (@$log_data_ref) {

    print "$0: DEBUG line: $line\n" if $DEBUG;

    #Expect data like this:

    #Uptime:      4 days, 17:53:59.25

    #Uptime:      1 hour, 15:56.71

    #Uptime:      2 hours, 00:28.96

    #Uptime:      4 minutes, 49.57

    # Or from cpu logs - not working yet

    # green Tue Feb 15 10:39:00 EST 2005 up: 17 day(s), 2 users, 79 procs, load=0.29

    # or with the patch above:
    # green Tue Feb 15 10:39:00 EST 2005 up: 17 day(s) 22:41, 2 users, 79 procs, load=0.29

    next unless $line =~ /^Uptime:/;
    if ( $line =~ /(\d+) days?, (\d\d)\:(\d\d)\:(\d\d)\.\d\d/ ) {
      $days_up = $1;
      $hours_up = $2;
      $minutes_up = $3;
      $seconds_up = $4;
      #$hundreth_seconds_up = $5;
    } elsif ($line =~ /(\d+) hours?, (\d\d)\:(\d\d)\.\d\d/) {
      $hours_up = $1;
      $minutes_up = $2;
      $seconds_up = $3;
      #$hundreth_seconds_up = $4;
    } elsif ($line =~ /(\d+) minutes, (\d\d)\.\d\d/) {
      $minutes_up = $1;
      $seconds_up = $2;
      #$hundreth_seconds_up = $3;
    } else {
      next;
    }
 
    if (defined ($days_up)) {
      $days_in_minutes = $days_up * 24 * 60;
    } else {
      $days_in_minutes = 0;
    }

    if (defined ($hours_up)) {
      if ($hours_up != 0) {
        $hours_in_minutes = $hours_up * 60;
      }
    } else {
      $hours_in_minutes = 0;
    }

    $total_minutes_up =  $days_in_minutes 
                       + $hours_in_minutes
                       + $minutes_up;


    print "$0: DEBUG total Minutes: $total_minutes_up\n" if $DEBUG;

    return $total_minutes_up;

  }
}

##############################################
# end of script
##############################################
